

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Three-state SERCA model %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% This function outputs the SERCA three-state model cycle rate 
% and phosphorylation proportion as a function of Cai, Casr, ATP, ADP and Pi
% All concentrations are in units of mM - including H_i

% Author: Kenneth Tran
% Last updated: 25/09/08


function [v_cycle, phosph] = Tran_SERCA_3state(Ca_i,Ca_sr,H_i,ATP,ADP,Pi)



        %Assigning the parameter values
        k_p1 = 25900;
        k_p2 = 2540;
        k_p3 = 20.5;
        k_m1 = 2;
        k_m2 = 67200;
        k_m3 = 149;
        
        kdcai = 0.91;
        kdcasr = 2.24;
        kdh1 = 1.09e-5;
        kdhi = 3.54e-3;
        kdhsr = 1.05e-8;
        kdh = 7.24e-5;
        n_H = 2.0;
        
        T_Cai = Ca_i/kdcai;
        T_Casr = Ca_sr/kdcasr;
        T_H1 = H_i/kdh1;
        T_Hi = (H_i^n_H)/kdhi;
        T_Hsr = (H_i.^n_H)/kdhsr;
        T_H = H_i/kdh;  

        % Apparent forward rate constants 
        a_p1 = k_p1*ATP;
        a_p2 = k_p2*T_Cai*T_Cai/(T_Cai^2 + T_Cai^2.*T_Hi + T_Hi*(1+T_H1));
        a_p3 = (k_p3*T_Hsr)/(T_Casr*T_Casr*T_H + T_H*(1) + T_Hsr*(1+T_H));

        % Apparent backward rate constants
        a_m1 = k_m1*T_Hi/(T_Cai^2 + T_Cai^2.*T_Hi + T_Hi*(1+T_H1));
        a_m2 = (k_m2*ADP*T_Casr*T_Casr*T_H)/(T_Casr*T_Casr*T_H + T_H + T_Hsr*(1+T_H));
        a_m3 = k_m3*Pi;

        % Calculating steady state flux using the diagram method
        s1 = a_p2*a_p3 + a_m1*a_p3 + a_m1*a_m2;
        s2 = a_p1*a_p3 + a_m2*a_p1 + a_m2*a_m3;
        s3 = a_p1*a_p2 + a_m3*a_m1 + a_m3*a_p2;
        
        sum = s1 + s2 + s3;
        
        % Calculating the cycle rate
        v_cycle = (a_p1*a_p2*a_p3 - a_m1*a_m2*a_m3)/sum;

        % Proportion of states that are phosphorylated
        phosph = (s3)/(s1+s2+s3);
 
        
        
        